<?php
/*
Plugin Name: Dreamhost Shared Hosting CPU Usage
Plugin URI: http://www.bernabauer.com/dhshcu
Description: This plugin works only @ Dreamhost's shared hosting. For more information check <a href="http://wiki.dreamhost.com/index.php/CPU_Resources_FAQ#Where_can_I_find_resource_reports.3F">DH wiki about CPU usage</a>. This plugins gets the CPU Usage from the file and displays it on the CPU Usage Manage Page.
Author: Bernardo Bauer
Version: 0.9.1
Author URI: http://www.bernabauer.com
*/

global $wpdb;

// Hook for adding admin menus
register_activation_hook("/wp-content/plugins/dhshcu/dhshcu.php", 'dhshcu_activate');
register_deactivation_hook("/wp-content/plugins/dhshcu/dhshcu.php", 'dhshcu_deactivate');
add_action('admin_menu', 'dhshcu_add_pages');
add_action('dhshcu_cron', 'dhshcu_core' );
add_filter('cron_schedules', 'more_reccurences');

load_plugin_textdomain("dhshcu", 'wp-content/plugins/dhshcu');

/***********************************************************************/
/* Add more names for wp-cron */
function more_reccurences() {
	return array(
	'weekly' => array('interval' => 604800, 'display' => 'Once Weekly'),
	'fortnightly' => array('interval' => 1209600, 'display' => 'Once Fortnightly'),
	'timly2' => array('interval' => 60, 'display' => 'timly2'),
	);
}

/***********************************************************************/
/* Things to be done when plugins is activated */
function dhshcu_activate() {

	global $wpdb;

	$table_name = $wpdb->prefix . "dhshcu_history";

	if($wpdb->get_var("show tables like '$table_name'") != $table_name)
	{

		$sql = "CREATE TABLE IF NOT EXISTS ".$table_name." (
					id mediumint(9) NOT NULL AUTO_INCREMENT,
					cpu smallint(6) NOT NULL,
					plugins smallint(6) NOT NULL,
					date date NOT NULL,
					UNIQUE KEY id (id),
					UNIQUE KEY date (date)
		);";

		require_once(ABSPATH . 'wp-admin/upgrade-functions.php');
		dbDelta($sql);
	}

	if (!wp_next_scheduled('dhshcu_cron')) {
		wp_schedule_event( mktime(8, 0, 0, date('m'), date('d'), date('Y')), 'daily', 'dhshcu_cron' );
	}

	$dhshcu_options = array('username'=>'','share'=>'', 'key'=>'', 'version'=>'0.9.1');

	add_option('dhshcu_options', $dhshcu_options);

}
/***********************************************************************/
/* Things to be done when plugin is deactivated */
function dhshcu_deactivate() {

	delete_option('dhshcu_options');
	
	if (wp_next_scheduled('dhshcu_cron')) {
		wp_clear_scheduled_hook('dhshcu_cron' );
	}
}

/***********************************************************************/
/* This function has been copyed from Akismet */
// Returns array with headers in $response[0] and body in $response[1]
function dhshcu_http_post($request, $host, $path, $port = 80) {
	global $wp_version;

	$dhshcu_options = get_option('dhshcu_options');

	$http_request  = "POST $path HTTP/1.0\r\n";
	$http_request .= "Host: $host\r\n";
	$http_request .= "Content-Type: application/x-www-form-urlencoded; charset=" . get_option('blog_charset') . "\r\n";
	$http_request .= "Content-Length: " . strlen($request) . "\r\n";
	$http_request .= "User-Agent: WordPress/$wp_version | DHSHCU/".$dhshcu_options['version']."\r\n";
	$http_request .= "\r\n";
	$http_request .= $request;

	$response = '';
	if( false != ( $fs = @fsockopen($host, $port, $errno, $errstr, 10) ) ) {
		fwrite($fs, $http_request);

		while ( !feof($fs) )
			$response .= fgets($fs, 1160); // One TCP-IP packet
		fclose($fs);
		$response = explode("\r\n\r\n", $response, 2);
	}
	return $response;
}

/***********************************************************************/
/* Send data to dhshcu HQ */
function dhshcu_send_data( $data ) {

	$blog = $data['blog'];
	$op = $data['op'];
	$key = $data['key'];
	$cpu = $data['cpu'];
	$plugins = $data['plugins'];
	$date = $data['date'];
	$email = $data['email'];
	$version = $data['version'];
	
	//send data
	$response = dhshcu_http_post("op=$op&key=$key&blog=$blog&cpu=$cpu&plugins=$plugins&date=$date&email=$email&version=$version", 'wp-plugins.bernabauer.com', '/dhshcu/index.php');

	//if all gets lost, pray for the gods!
	if ( !is_array($response) || !isset($response[1]) || $response[1] != 'ok' && $response[1] != 'nok' ) {
		return $response[1];
	}
	//urray! this stuff really works!
	return $response[1];
}


/***********************************************************************/
/* Count or list the active plugins */
/* if $param - 'list' the result is a list otherwise the result is the count */
function dhshcu_getactiveplugins($param) {

	global $wpdb;
	require_once(ABSPATH . 'wp-admin/admin-functions.php');

	$i = 0;
	if ( get_option('active_plugins') )
		$current_plugins = get_option('active_plugins');

	$plugins = get_plugins();
	
	foreach($plugins as $plugin_file => $plugin_data) {
		if (!empty($current_plugins) && in_array($plugin_file, $current_plugins)) {
			$i++;
			$plugins_allowedtags = array('a' => array('href' => array(),'title' => array()),'abbr' => array('title' => array()),'acronym' => array('title' => array()),'code' => array(),'em' => array(),'strong' => array());
	
			// Sanitize all displayed data
			$plugin_data['Title']       = wp_kses($plugin_data['Title'], $plugins_allowedtags);
			$plugin_data['Version']     = wp_kses($plugin_data['Version'], $plugins_allowedtags);
			$plugin_data['Description'] = wp_kses($plugin_data['Description'], $plugins_allowedtags);
			$plugin_data['Author']      = wp_kses($plugin_data['Author'], $plugins_allowedtags);
			
			$list .= "
		<tr>
			<td class='name'>{$plugin_data['Title']}</td>
			<td class='vers'>{$plugin_data['Version']}</td>
		</tr>";
	
		}		
	}


	if ($param == 'list') {
		return $list;
	} else {
		return $i;
	}
}

/***********************************************************************/
function dhshcu_core() {

	require_once(ABSPATH . 'wp-admin/admin-functions.php');
	global $wpdb;
	$domain = "dhshcu";
	$table_name = $wpdb->prefix . "dhshcu_history";
	$dhshcu_options = get_option('dhshcu_options');
	
	//populate array with some data do send
	$datatosend['op'] = "senddata";
	$datatosend['blog'] = urlencode( get_option('home') );
	$datatosend['plugins'] = dhshcu_getactiveplugins('numb');
	$datatosend['email'] = urlencode( get_option('admin_email') );
	$key = $dhshcu_options['key'];
	$datatosend['key'] = $key;
	$datatosend['cpu'] = dhshcu_getcpuusage();
	$datatosend['version'] = $dhshcu_options['version'];

	//mount the date. Data collected if from yesterday, so...
	$datatosend['date'] =  date('Y-m-d',mktime(8, 0, 0, date('m'), date('d')-1, date('Y')));


	//insert data into database			
	$insert = "INSERT INTO " . $table_name .
				" (cpu, date, plugins) " .
				"VALUES ('" . $datatosend['cpu'] . "', '" . $datatosend['date'] . "', '" . $datatosend['plugins'] . "' )";
	
	$results = $wpdb->query( $insert );
	
	//send data to the collective (Yes! We're Borg! Resistance is futile!)
#	if ($dhshcu_options["share"] != '') {
#		$result2 = dhshcu_send_data($datatosend);
#	}	

}

/***********************************************************************/
/* get cpu count from file */
function dhshcu_getcpuusage() {

	$dhshcu_options = get_option('dhshcu_options');
	
	//if username is not set the plugin can't get anything done
	if ($dhshcu_options["username"] != '') {

		//get the file with the cpu usage
		$file = file_get_contents ('/home/'.$dhshcu_options["username"].'/logs/resources/'. $dhshcu_options["username"].'.sa.analyzed.0');
		
		//if the file has something to be processed do the parsing to get the cpu usage
		if ($file != '') {
			$pattern = '/Total:\s*[0-9]*/';
			preg_match($pattern, $file, $matches);
			$cpu = substr($matches[0], 7);
			
		} else {
			//the file does not exist. I wonder what went wrong...
			$cpu = __("Error getting CPU usage.",$domain);
		}
	}

	return $cpu;
}
/***********************************************************************/
/* Adds the menus to manage and configure the plugin */
function dhshcu_add_pages() {

	$domain = "dhshcu";

    // Add a new submenu under Options:
    if ( function_exists('add_options_page') ) {
		add_options_page(__('CPU Usage',$domain), __('CPU Usage',$domain), 8, 'dhshcu.php', 'dhshcu_options_page');
    }
    // Add a new submenu under Manage:
    if ( function_exists('add_management_page') ) {
		add_management_page(__('CPU Usage',$domain), __('CPU Usage',$domain), 8, 'dhshcu.php', 'dhshcu_manage_page');
    }
}

/***********************************************************************/
/* The warning code */
if (  !isset($_POST['info_update']) ) {
	function dhshcu_warning() {
		$domain = "dhshcu";

		$dhshcu_options = get_option('dhshcu_options');

		if ( $dhshcu_options['username'] == '') {
			$msg .= "* ".sprintf(__('Please <a href="%1$s">set</a> your Dreamhost username.',$domain), "options-general.php?page=dhshcu.php")."<br />";
		} else {
			if (!file_exists ('/home/'. $dhshcu_options["username"].'/logs/resources/'. $dhshcu_options["username"].'.sa.analyzed.0')) {
				$msg .= "<strong>* ".__('You have to activate CPU monitoring.',$domain)."</strong> ".sprintf(__('Check out how <a href="%1$s" target="_blank">here</a>.',$domain), "http://wiki.dreamhost.com/index.php/CPU_Resources_FAQ#Where_can_I_find_resource_reports.3F");
			}	
		}
		if ($msg) 
			echo "<div class='updated fade-ff0000'><p><strong>".__('Dreamhost Shared Hosting CPU Usage:',$domain)."</strong><br /> ".$msg."</p></div>";
	}
	add_action('admin_notices', 'dhshcu_warning');
	return;
}
/***********************************************************************/
/* Manage page is rendered here */
function dhshcu_manage_page() {

	global $wpdb;
	$domain = 'dhshcu';
	$table_name = $wpdb->prefix . "dhshcu_history";
	$dhshcu_options = get_option('dhshcu_options');

	?>
	<div class="wrap">
		<h2><?php _e('Last 7 days CPU usage',$domain); ?></h2>

		<?php
		
			$usage_history = $wpdb->get_results("SELECT date,cpu,plugins FROM $table_name WHERE date > DATE_SUB(CURDATE(), INTERVAL 7 DAY) ORDER BY date DESC");
			
			echo "<table><tr><th>". __('Date',$domain)."</th><th>". __('CPU Seconds',$domain)."</th><th>". __('Plugins',$domain)."</th></tr>";
			if ($usage_history) {
				$numb = 0;
				foreach ($usage_history as $usage_forday) {
					if ($numb < 7) {
						echo "<tr><td><a href='http://".$_SERVER['SERVER_NAME']."/stats/resources/".$dhshcu_options['username'].".sa.analyzed.".$numb."'>".$usage_forday->date."</a></td><td><center>".$usage_forday->cpu."</center></td><td><center>".$usage_forday->plugins."</center></td></tr>";
						$numb++;
					} else
						echo $usage_forday->date." - ".$usage_forday->cpu."<BR>";
				}
			} else {
				echo __("No data has been collected yet.",$domain)."<br><br>";
			}
			echo "</table><br /><br />".__("If the direct link to raw CPU usage does not work, you should read <a href='http://wiki.dreamhost.com/index.php/Making_stats_accessible_with_htaccess'>this</a>.",$domain)."<br /><br />";

			//GET Difference between Server TZ and desired TZ
			$sec_diff = date('Z') - (get_option('gmt_offset') * 3600);
			$sec_diff=(($sec_diff<=0)?'+':'-').abs($sec_diff);			

			echo "<br />".__('Next CPU usage fetch :', $domain)." ". date('d/m/Y H:i:s', wp_next_scheduled('dhshcu_cron') + $sec_diff); 
		?>

			<h2><?php _e('About', $domain); ?></h2>

			<p><?php _e('This plugin was developed to ease the job of fellow users who have wordpress blogs hosted at Dreamhost. Shared hosting maybe cheap, but has some guidelines that should be followed.One of them is the CPU Usage. Dreamhost offers a way to watch how much CPU your blog (or anything else) uses. You have to <a href="http://wiki.dreamhost.com/index.php/CPU_Resources_FAQ#Where_can_I_find_resource_reports.3F">enable this feature</a> to use this plugin.', $domain); ?></p>
			
			<p><?php _e('After a while trying to make my blog "sharedhosting-CPU-friendly", I realized that i was wasting precious time watching the daily statistics. Everyday I logged in via Telnet to get the numbers. I got tired of it and decided to waste my time making this plugin.', $domain); ?></p>
			
			<p><?php _e("I should tell you that I'm <strong>*NOT*</strong> affiliated to Dreamhost nor work for them. As much as you, I host my blogs using sharedhosting and if you like my work, you can express your gratitude donating.", $domain); ?></p>
			
			<p><?php _e('Any thoughts or suggestions can be made using my <a href="http://www.bernabauer.com/contato">contact form</a>. The form is in portuguese, but I think you wont have any problems. ;-)', $domain); ?></p>
	</div>
<?php 
}
/***********************************************************************/
/* The Options page is rendered here */
function dhshcu_options_page() {

	global $wpdb;
	$domain = 'dhshcu';
	$table_name = $wpdb->prefix . "dhshcu_history";
	$dhshcu_options = get_option('dhshcu_options');

    if ( isset($_POST['genkey']) ) {
			$datatosend['op'] = "genkey";
			$datatosend['blog'] = get_option('home');
			$datatosend['email'] = get_option('admin_email');
		
			$result = dhshcu_send_data( $datatosend );
		?>
			<div class="updated"><p><strong><?php _e('The Key has been sent to your email.',$domain); ?></strong></p></div>
		<?php 
	}

    if ( isset($_POST['info_update']) ) {

		if (isset($_POST['dhshcu_share'])) {
			$dhshcu_options['share'] = "checked";
		} else
			$dhshcu_options['share'] = "";

		$dhshcu_options['username'] = $_POST['dhshcu_username'];
		$dhshcu_options['key'] = $_POST['dhshcu_key'];

		update_option('dhshcu_options', $dhshcu_options);
		

		?>
			<div class="updated"><p><strong><?php _e('Your options have been updated.',$domain); ?></strong></p></div>
		<?php 

    }

	?>

	<div class="wrap">
			<h2><?php _e('Configuration', $domain); ?></h2>
		
			<form method="post">
		
				<p><?php _e('Your Dreamhost username:', $domain); ?> <input name="dhshcu_username" type="text" id="dhshcu_username" value="<?php $username = $dhshcu_options['username']; echo $username; ?>" size=15  />
				<br /><?php _e('This is the username under which your blog runs.', $domain); ?></p>

<!--
				<p><input name="dhshcu_share" type="checkbox" id="dhshcu_share" <?php if($dhshcu_options['share'] == 'checked') { echo 'checked'; } ?> /> <?php _e('Share CPU Usage Data', $domain); ?>
				<br /><?php _e('Enabling this option your CPU usage will be sent to the author and you will be able to see other blog\'s CPU Usage.', $domain); ?><br />
				DHSHCU Key : <input name="dhshcu_key" type="text" id="dhshcu_key" value="<?php $key = $dhshcu_options['key']; echo $key; ?>" size=57  />
				</p>
-->
				<div class="submit">
				  <input type="submit" name="info_update" value="<?php _e('Update', $domain); ?>" />
				</div>
			</form> 
<!--
			<form method="post"><input type="submit" name="genkey" value="<?php _e('Generate Key', $domain); ?>" /></form>
-->
			<h2><?php _e('About', $domain); ?></h2>

			<p><?php _e('This plugin was developed to ease the job of fellow users who have wordpress blogs hosted at Dreamhost. Shared hosting maybe cheap, but has some guidelines that should be followed.One of them is the CPU Usage. Dreamhost offers a way to watch how much CPU your blog (or anything else) uses. You have to <a href="http://wiki.dreamhost.com/index.php/CPU_Resources_FAQ#Where_can_I_find_resource_reports.3F">enable this feature</a> to use this plugin.', $domain); ?></p>
			
			<p><?php _e('After a while trying to make my blog "sharedhosting-CPU-friendly", I realized that i was wasting precious time watching the daily statistics. Everyday I logged in via Telnet to get the numbers. I got tired of it and decided to waste my time making this plugin.', $domain); ?></p>
			
			<p><?php _e("I should tell you that I'm <strong>*NOT*</strong> affiliated to Dreamhost nor work for them. As much as you, I host my blogs using sharedhosting and if you like my work, you can express your gratitude donating or enabling the CPU Usage Badge.", $domain); ?></p>
			
			<p><?php _e('Any thoughts or suggestions can be made using my <a href="http://www.bernabauer.com/contato">contact form</a>. The form is in portuguese, but I think you wont have any problems. ;-)', $domain); ?></p>
	</div>
<?php 
}
?>